@@ -1,54 +1,26 @@
 /*
- * Copyright (C) 2010, 2013 Chris Aniszczyk <caniszczyk@gmail.com>
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2010, 2013 Chris Aniszczyk <caniszczyk@gmail.com> and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 package org.eclipse.jgit.api;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotEquals;
+import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
 
+import java.io.File;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
 
+import org.eclipse.jgit.junit.JGitTestUtil;
 import org.eclipse.jgit.junit.RepositoryTestCase;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.ObjectId;
@@ -104,6 +76,25 @@ public class FetchCommandTest extends RepositoryTestCase {
 				db.resolve(tagRef.getObjectId().getName()));
 	}
 
+	@Test
+	public void testForcedFetch() throws Exception {
+		remoteGit.commit().setMessage("commit").call();
+		remoteGit.commit().setMessage("commit2").call();
+		git.fetch().setRemote("test")
+				.setRefSpecs("refs/heads/master:refs/heads/master").call();
+
+		remoteGit.commit().setAmend(true).setMessage("amended").call();
+		FetchResult res = git.fetch().setRemote("test")
+				.setRefSpecs("refs/heads/master:refs/heads/master").call();
+		assertEquals(RefUpdate.Result.REJECTED,
+				res.getTrackingRefUpdate("refs/heads/master").getResult());
+		res = git.fetch().setRemote("test")
+				.setRefSpecs("refs/heads/master:refs/heads/master")
+				.setForceUpdate(true).call();
+		assertEquals(RefUpdate.Result.FORCED,
+				res.getTrackingRefUpdate("refs/heads/master").getResult());
+	}
+
 	@Test
 	public void fetchAddsBranches() throws Exception {
 		final String branch1 = "b1";
@@ -274,6 +265,45 @@ public class FetchCommandTest extends RepositoryTestCase {
 		assertEquals(tagRef2.getObjectId(), db.resolve(tagName));
 	}
 
+	@Test
+	public void testFetchWithPruneShouldKeepOriginHead() throws Exception {
+		// Create a commit in the test repo.
+		commitFile("foo", "foo", "master");
+		// Produce a real clone of the git repo
+		Git cloned = Git.cloneRepository()
+				.setDirectory(createTempDirectory("testCloneRepository"))
+				.setURI("file://"
+						+ git.getRepository().getWorkTree().getAbsolutePath())
+				.call();
+		assertNotNull(cloned);
+		Repository clonedRepo = cloned.getRepository();
+		addRepoToClose(clonedRepo);
+		ObjectId originMasterId = clonedRepo
+				.resolve("refs/remotes/origin/master");
+		assertNotNull("Should have origin/master", originMasterId);
+		assertNotEquals("origin/master should not be zero ID",
+				ObjectId.zeroId(), originMasterId);
+		// Canonical git creates origin/HEAD; JGit (for now) doesn't. Let's
+		// pretend we did the clone via command-line git.
+		ObjectId originHeadId = clonedRepo.resolve("refs/remotes/origin/HEAD");
+		if (originHeadId == null) {
+			JGitTestUtil.write(
+					new File(clonedRepo.getDirectory(),
+							"refs/remotes/origin/HEAD"),
+					"ref: refs/remotes/origin/master\n");
+			originHeadId = clonedRepo.resolve("refs/remotes/origin/HEAD");
+		}
+		assertEquals("Should have origin/HEAD", originMasterId, originHeadId);
+		FetchResult result = cloned.fetch().setRemote("origin")
+				.setRemoveDeletedRefs(true).call();
+		assertTrue("Fetch after clone should be up-to-date",
+				result.getTrackingRefUpdates().isEmpty());
+		assertEquals("origin/master should still exist", originMasterId,
+				clonedRepo.resolve("refs/remotes/origin/master"));
+		assertEquals("origin/HEAD should be unchanged", originHeadId,
+				clonedRepo.resolve("refs/remotes/origin/HEAD"));
+	}
+
 	@Test
 	public void fetchAddRefsWithDuplicateRefspec() throws Exception {
 		final String branchName = "branch";
