@@ -1,44 +1,11 @@
 /*
- * Copyright (C) 2010, Chris Aniszczyk <caniszczyk@gmail.com>
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2010, Chris Aniszczyk <caniszczyk@gmail.com> and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 package org.eclipse.jgit.api;
 
@@ -105,6 +72,10 @@ public class FetchCommand extends TransportCommand<FetchCommand, FetchResult> {
 
 	private Callback callback;
 
+	private boolean isForceUpdate;
+
+	private String initialBranch;
+
 	/**
 	 * Callback for status of fetch operation.
 	 *
@@ -191,14 +162,16 @@ public class FetchCommand extends TransportCommand<FetchCommand, FetchResult> {
 					// updated to an object that is not currently present in the
 					// submodule.
 					if ((recurseMode == FetchRecurseSubmodulesMode.ON_DEMAND
-							&& !submoduleRepo.hasObject(walk.getObjectId()))
+							&& !submoduleRepo.getObjectDatabase()
+									.has(walk.getObjectId()))
 							|| recurseMode == FetchRecurseSubmodulesMode.YES) {
 						FetchCommand f = new FetchCommand(submoduleRepo)
 								.setProgressMonitor(monitor)
 								.setTagOpt(tagOption)
 								.setCheckFetchedObjects(checkFetchedObjects)
 								.setRemoveDeletedRefs(isRemoveDeletedRefs())
-								.setThin(thin).setRefSpecs(refSpecs)
+								.setThin(thin)
+								.setRefSpecs(applyOptions(refSpecs))
 								.setDryRun(dryRun)
 								.setRecurseSubmodules(recurseMode);
 						configure(f);
@@ -237,8 +210,8 @@ public class FetchCommand extends TransportCommand<FetchCommand, FetchResult> {
 				transport.setTagOpt(tagOption);
 			transport.setFetchThin(thin);
 			configure(transport);
-
-			FetchResult result = transport.fetch(monitor, refSpecs);
+			FetchResult result = transport.fetch(monitor,
+					applyOptions(refSpecs), initialBranch);
 			if (!repo.isBare()) {
 				fetchSubmodules(result);
 			}
@@ -252,7 +225,7 @@ public class FetchCommand extends TransportCommand<FetchCommand, FetchResult> {
 					e.getMessage(), e);
 		} catch (URISyntaxException e) {
 			throw new InvalidRemoteException(MessageFormat.format(
-					JGitText.get().invalidRemote, remote));
+					JGitText.get().invalidRemote, remote), e);
 		} catch (NotSupportedException e) {
 			throw new JGitInternalException(
 					JGitText.get().exceptionCaughtDuringExecutionOfFetchCommand,
@@ -261,6 +234,17 @@ public class FetchCommand extends TransportCommand<FetchCommand, FetchResult> {
 
 	}
 
+	private List<RefSpec> applyOptions(List<RefSpec> refSpecs2) {
+		if (!isForceUpdate()) {
+			return refSpecs2;
+		}
+		List<RefSpec> updated = new ArrayList<>(3);
+		for (RefSpec refSpec : refSpecs2) {
+			updated.add(refSpec.setForceUpdate(true));
+		}
+		return updated;
+	}
+
 	/**
 	 * Set the mode to be used for recursing into submodules.
 	 *
@@ -345,17 +329,17 @@ public class FetchCommand extends TransportCommand<FetchCommand, FetchResult> {
 	 * @return whether to remove refs which no longer exist in the source
 	 */
 	public boolean isRemoveDeletedRefs() {
-		if (removeDeletedRefs != null)
+		if (removeDeletedRefs != null) {
 			return removeDeletedRefs.booleanValue();
-		else { // fall back to configuration
-			boolean result = false;
-			StoredConfig config = repo.getConfig();
-			result = config.getBoolean(ConfigConstants.CONFIG_FETCH_SECTION,
-					null, ConfigConstants.CONFIG_KEY_PRUNE, result);
-			result = config.getBoolean(ConfigConstants.CONFIG_REMOTE_SECTION,
-					remote, ConfigConstants.CONFIG_KEY_PRUNE, result);
-			return result;
 		}
+		// fall back to configuration
+		boolean result = false;
+		StoredConfig config = repo.getConfig();
+		result = config.getBoolean(ConfigConstants.CONFIG_FETCH_SECTION, null,
+				ConfigConstants.CONFIG_KEY_PRUNE, result);
+		result = config.getBoolean(ConfigConstants.CONFIG_REMOTE_SECTION,
+				remote, ConfigConstants.CONFIG_KEY_PRUNE, result);
+		return result;
 	}
 
 	/**
@@ -505,6 +489,24 @@ public class FetchCommand extends TransportCommand<FetchCommand, FetchResult> {
 		return this;
 	}
 
+	/**
+	 * Set the initial branch
+	 *
+	 * @param branch
+	 *            the initial branch to check out when cloning the repository.
+	 *            Can be specified as ref name (<code>refs/heads/master</code>),
+	 *            branch name (<code>master</code>) or tag name
+	 *            (<code>v1.2.3</code>). The default is to use the branch
+	 *            pointed to by the cloned repository's HEAD and can be
+	 *            requested by passing {@code null} or <code>HEAD</code>.
+	 * @return {@code this}
+	 * @since 5.11
+	 */
+	public FetchCommand setInitialBranch(String branch) {
+		this.initialBranch = branch;
+		return this;
+	}
+
 	/**
 	 * Register a progress callback.
 	 *
@@ -517,4 +519,27 @@ public class FetchCommand extends TransportCommand<FetchCommand, FetchResult> {
 		this.callback = callback;
 		return this;
 	}
+
+	/**
+	 * Whether fetch --force option is enabled
+	 *
+	 * @return whether refs affected by the fetch are updated forcefully
+	 * @since 5.0
+	 */
+	public boolean isForceUpdate() {
+		return this.isForceUpdate;
+	}
+
+	/**
+	 * Set fetch --force option
+	 *
+	 * @param force
+	 *            whether to update refs affected by the fetch forcefully
+	 * @return this command
+	 * @since 5.0
+	 */
+	public FetchCommand setForceUpdate(boolean force) {
+		this.isForceUpdate = force;
+		return this;
+	}
 }
