@@ -1,44 +1,11 @@
 /*
- * Copyright (C) 2011, GitHub Inc.
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2011, GitHub Inc. and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 package org.eclipse.jgit.api;
 
@@ -79,7 +46,7 @@ public class SubmoduleSyncCommand extends GitCommand<Map<String, String>> {
 	 * @param repo
 	 *            a {@link org.eclipse.jgit.lib.Repository} object.
 	 */
-	public SubmoduleSyncCommand(final Repository repo) {
+	public SubmoduleSyncCommand(Repository repo) {
 		super(repo);
 		paths = new ArrayList<>();
 	}
@@ -91,7 +58,7 @@ public class SubmoduleSyncCommand extends GitCommand<Map<String, String>> {
 	 *            (with <code>/</code> as separator)
 	 * @return this command
 	 */
-	public SubmoduleSyncCommand addPath(final String path) {
+	public SubmoduleSyncCommand addPath(String path) {
 		paths.add(path);
 		return this;
 	}
@@ -104,12 +71,12 @@ public class SubmoduleSyncCommand extends GitCommand<Map<String, String>> {
 	 * @return shortened branch name, null on failures
 	 * @throws java.io.IOException
 	 */
-	protected String getHeadBranch(final Repository subRepo) throws IOException {
+	protected String getHeadBranch(Repository subRepo) throws IOException {
 		Ref head = subRepo.exactRef(Constants.HEAD);
-		if (head != null && head.isSymbolic())
+		if (head != null && head.isSymbolic()) {
 			return Repository.shortenRefName(head.getLeaf().getName());
-		else
-			return null;
+		}
+		return null;
 	}
 
 	/** {@inheritDoc} */
@@ -132,38 +99,37 @@ public class SubmoduleSyncCommand extends GitCommand<Map<String, String>> {
 						path, ConfigConstants.CONFIG_KEY_URL, remoteUrl);
 				synced.put(path, remoteUrl);
 
-				Repository subRepo = generator.getRepository();
-				if (subRepo == null)
-					continue;
+				try (Repository subRepo = generator.getRepository()) {
+					if (subRepo == null) {
+						continue;
+					}
+
+					StoredConfig subConfig;
+					String branch;
 
-				StoredConfig subConfig;
-				String branch;
-				try {
 					subConfig = subRepo.getConfig();
 					// Get name of remote associated with current branch and
 					// fall back to default remote name as last resort
 					branch = getHeadBranch(subRepo);
 					String remote = null;
-					if (branch != null)
+					if (branch != null) {
 						remote = subConfig.getString(
 								ConfigConstants.CONFIG_BRANCH_SECTION, branch,
 								ConfigConstants.CONFIG_KEY_REMOTE);
-					if (remote == null)
+					}
+					if (remote == null) {
 						remote = Constants.DEFAULT_REMOTE_NAME;
+					}
 
 					subConfig.setString(ConfigConstants.CONFIG_REMOTE_SECTION,
 							remote, ConfigConstants.CONFIG_KEY_URL, remoteUrl);
 					subConfig.save();
-				} finally {
-					subRepo.close();
 				}
 			}
 			if (!synced.isEmpty())
 				config.save();
 			return synced;
-		} catch (IOException e) {
-			throw new JGitInternalException(e.getMessage(), e);
-		} catch (ConfigInvalidException e) {
+		} catch (IOException | ConfigInvalidException e) {
 			throw new JGitInternalException(e.getMessage(), e);
 		}
 	}
